@@ -108,7 +108,7 @@
 --
 --   * ABC from University of Berkeley: <http://www.eecs.berkeley.edu/~alanmi/abc/>
 --
---   * CVC4 from Stanford: <http://cvc4.cs.stanford.edu/web/>
+--   * CVC4 from Stanford: <https://cvc4.github.io/>
 --
 --   * Boolector from Johannes Kepler University: <http://fmv.jku.at/boolector/>
 --
@@ -156,7 +156,7 @@ module Data.SBV (
   , SFloat, SDouble
   -- ** Algebraic reals
   -- $algReals
-  , SReal, AlgReal, sRealToSInteger
+  , SReal, AlgReal, sRealToSInteger, algRealToRational
   -- ** Characters, Strings and Regular Expressions
   -- $strings
   , SChar, SString
@@ -242,7 +242,7 @@ module Data.SBV (
   -- $enumerations
   , mkSymbolicEnumeration
 
-  -- * Uninterpreted sorts, constants, and functions
+  -- * Uninterpreted sorts, axioms, constants, and functions
   -- $uninterpreted
   , Uninterpreted(..), addAxiom
 
@@ -255,6 +255,7 @@ module Data.SBV (
   , satWith, allSat, allSatWith, optimize, optimizeWith, isVacuous
   , isVacuousWith, isTheorem, isTheoremWith, isSatisfiable, isSatisfiableWith
   , proveWithAll, proveWithAny, satWithAll
+  , proveConcurrentWithAny, proveConcurrentWithAll, satConcurrentWithAny, satConcurrentWithAll
   , satWithAny, generateSMTBenchmark
   , solve
   -- * Constraints
@@ -346,7 +347,7 @@ module Data.SBV (
   ) where
 
 import Data.SBV.Core.AlgReals
-import Data.SBV.Core.Data       hiding (addAxiom, forall, forall_,
+import Data.SBV.Core.Data       hiding (forall, forall_,
                                         mkForallVars, exists, exists_,
                                         mkExistVars, free, free_, mkFreeVars,
                                         output, symbolic, symbolics, mkSymVal,
