@@ -1,10 +1,10 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Compilers.CodeGen
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Compilers.CodeGen
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Code generation utilities
 -----------------------------------------------------------------------------
@@ -13,6 +13,8 @@
 {-# LANGUAGE FlexibleInstances          #-}
 {-# LANGUAGE GeneralizedNewtypeDeriving #-}
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Data.SBV.Compilers.CodeGen (
         -- * The codegen monad
           SBVCodeGen(..), cgSym
@@ -29,7 +31,7 @@ module Data.SBV.Compilers.CodeGen (
 
         -- * Settings
         , cgPerformRTCs, cgSetDriverValues
-        , cgAddPrototype, cgAddDecl, cgAddLDFlags, cgIgnoreSAssert, cgOverwriteFiles
+        , cgAddPrototype, cgAddDecl, cgAddLDFlags, cgIgnoreSAssert, cgOverwriteFiles, cgShowU8UsingHex
         , cgIntegerSize, cgSRealType, CgSRealType(..)
 
         -- * Infrastructure
@@ -41,8 +43,7 @@ module Data.SBV.Compilers.CodeGen (
         ) where
 
 import Control.Monad             (filterM, replicateM, unless)
-import Control.Monad.Reader      (ask)
-import Control.Monad.Trans       (MonadIO, lift, liftIO)
+import Control.Monad.Trans       (MonadIO(liftIO), lift)
 import Control.Monad.State.Lazy  (MonadState, StateT(..), modify')
 import Data.Char                 (toLower, isSpace)
 import Data.List                 (nub, isPrefixOf, intercalate, (\\))
@@ -54,7 +55,7 @@ import           Text.PrettyPrint.HughesPJ      (Doc, vcat)
 import qualified Text.PrettyPrint.HughesPJ as P (render)
 
 import Data.SBV.Core.Data
-import Data.SBV.Core.Symbolic (svToSymSW, svMkSymVar, outputSVal)
+import Data.SBV.Core.Symbolic (MonadSymbolic(..), svToSymSV, svMkSymVar, outputSVal)
 
 #if MIN_VERSION_base(4,11,0)
 import Control.Monad.Fail as Fail
@@ -75,6 +76,7 @@ data CgConfig = CgConfig {
         , cgGenMakefile        :: Bool               -- ^ If 'True', will generate a makefile
         , cgIgnoreAsserts      :: Bool               -- ^ If 'True', will ignore 'Data.SBV.sAssert' calls
         , cgOverwriteGenerated :: Bool               -- ^ If 'True', will overwrite the generated files without prompting.
+        , cgShowU8InHex        :: Bool               -- ^ If 'True', then 8-bit unsigned values will be shown in hex as well, otherwise decimal. (Other types always shown in hex.)
         }
 
 -- | Default options for code generation. The run-time checks are turned-off, and the driver values are completely random.
@@ -87,11 +89,12 @@ defaultCgConfig = CgConfig { cgRTC                = False
                            , cgGenMakefile        = True
                            , cgIgnoreAsserts      = False
                            , cgOverwriteGenerated = False
+                           , cgShowU8InHex        = False
                            }
 
 -- | Abstraction of target language values
-data CgVal = CgAtomic SW
-           | CgArray  [SW]
+data CgVal = CgAtomic SV
+           | CgArray  [SV]
 
 -- | Code-generation state
 data CgState = CgState {
@@ -121,6 +124,7 @@ initCgState = CgState {
 -- and return values.
 newtype SBVCodeGen a = SBVCodeGen (StateT CgState Symbolic a)
                    deriving ( Applicative, Functor, Monad, MonadIO, MonadState CgState
+                            , MonadSymbolic
 #if MIN_VERSION_base(4,11,0)
                             , Fail.MonadFail
 #endif
@@ -130,10 +134,6 @@ newtype SBVCodeGen a = SBVCodeGen (StateT CgState Symbolic a)
 cgSym :: Symbolic a -> SBVCodeGen a
 cgSym = SBVCodeGen . lift
 
--- | Reach into symbolic monad and output a value. Returns the corresponding SW
-cgSBVToSW :: SBV a -> SBVCodeGen SW
-cgSBVToSW = cgSym . sbvToSymSW
-
 -- | Sets RTC (run-time-checks) for index-out-of-bounds, shift-with-large value etc. on/off. Default: 'False'.
 cgPerformRTCs :: Bool -> SBVCodeGen ()
 cgPerformRTCs b = modify' (\s -> s { cgFinalConfig = (cgFinalConfig s) { cgRTC = b } })
@@ -200,6 +200,12 @@ cgAddPrototype ss = modify' (\s -> let old = cgPrototypes s
 cgOverwriteFiles :: Bool -> SBVCodeGen ()
 cgOverwriteFiles b = modify' (\s -> s { cgFinalConfig = (cgFinalConfig s) { cgOverwriteGenerated = b } })
 
+-- | If passed 'True', then we will show 'SWord 8' type in hex. Otherwise we'll show it in decimal. All signed
+-- types are shown decimal, and all unsigned larger types are shown hexadecimal otherwise.
+cgShowU8UsingHex :: Bool -> SBVCodeGen ()
+cgShowU8UsingHex b = modify' (\s -> s { cgFinalConfig = (cgFinalConfig s) { cgShowU8InHex = b } })
+
+
 -- | Adds the given lines to the program file generated, useful for generating programs with uninterpreted functions.
 cgAddDecl :: [String] -> SBVCodeGen ()
 cgAddDecl ss = modify' (\s -> s { cgDecls = cgDecls s ++ ss })
@@ -210,93 +216,93 @@ cgAddLDFlags ss = modify' (\s -> s { cgLDFlags = cgLDFlags s ++ ss })
 
 -- | Creates an atomic input in the generated code.
 svCgInput :: Kind -> String -> SBVCodeGen SVal
-svCgInput k nm = do r  <- cgSym (ask >>= liftIO . svMkSymVar (Just ALL) k Nothing)
-                    sw <- cgSym (svToSymSW r)
-                    modify' (\s -> s { cgInputs = (nm, CgAtomic sw) : cgInputs s })
+svCgInput k nm = do r  <- symbolicEnv >>= liftIO . svMkSymVar (Just ALL) k Nothing
+                    sv <- svToSymSV r
+                    modify' (\s -> s { cgInputs = (nm, CgAtomic sv) : cgInputs s })
                     return r
 
 -- | Creates an array input in the generated code.
 svCgInputArr :: Kind -> Int -> String -> SBVCodeGen [SVal]
 svCgInputArr k sz nm
   | sz < 1 = error $ "SBV.cgInputArr: Array inputs must have at least one element, given " ++ show sz ++ " for " ++ show nm
-  | True   = do rs  <- cgSym $ ask >>= liftIO . replicateM sz . svMkSymVar (Just ALL) k Nothing
-                sws <- cgSym $ mapM svToSymSW rs
+  | True   = do rs  <- symbolicEnv >>= liftIO . replicateM sz . svMkSymVar (Just ALL) k Nothing
+                sws <- mapM svToSymSV rs
                 modify' (\s -> s { cgInputs = (nm, CgArray sws) : cgInputs s })
                 return rs
 
 -- | Creates an atomic output in the generated code.
 svCgOutput :: String -> SVal -> SBVCodeGen ()
-svCgOutput nm v = do _ <- cgSym (outputSVal v)
-                     sw <- cgSym (svToSymSW v)
-                     modify' (\s -> s { cgOutputs = (nm, CgAtomic sw) : cgOutputs s })
+svCgOutput nm v = do _ <- outputSVal v
+                     sv <- svToSymSV v
+                     modify' (\s -> s { cgOutputs = (nm, CgAtomic sv) : cgOutputs s })
 
 -- | Creates an array output in the generated code.
 svCgOutputArr :: String -> [SVal] -> SBVCodeGen ()
 svCgOutputArr nm vs
   | sz < 1 = error $ "SBV.cgOutputArr: Array outputs must have at least one element, received " ++ show sz ++ " for " ++ show nm
-  | True   = do _ <- cgSym (mapM outputSVal vs)
-                sws <- cgSym (mapM svToSymSW vs)
+  | True   = do mapM_ outputSVal vs
+                sws <- mapM svToSymSV vs
                 modify' (\s -> s { cgOutputs = (nm, CgArray sws) : cgOutputs s })
   where sz = length vs
 
 -- | Creates a returned (unnamed) value in the generated code.
 svCgReturn :: SVal -> SBVCodeGen ()
-svCgReturn v = do _ <- cgSym (outputSVal v)
-                  sw <- cgSym (svToSymSW v)
-                  modify' (\s -> s { cgReturns = CgAtomic sw : cgReturns s })
+svCgReturn v = do _ <- outputSVal v
+                  sv <- svToSymSV v
+                  modify' (\s -> s { cgReturns = CgAtomic sv : cgReturns s })
 
 -- | Creates a returned (unnamed) array value in the generated code.
 svCgReturnArr :: [SVal] -> SBVCodeGen ()
 svCgReturnArr vs
   | sz < 1 = error $ "SBV.cgReturnArr: Array returns must have at least one element, received " ++ show sz
-  | True   = do _ <- cgSym (mapM outputSVal vs)
-                sws <- cgSym (mapM svToSymSW vs)
+  | True   = do mapM_ outputSVal vs
+                sws <- mapM svToSymSV vs
                 modify' (\s -> s { cgReturns = CgArray sws : cgReturns s })
   where sz = length vs
 
 -- | Creates an atomic input in the generated code.
-cgInput :: SymWord a => String -> SBVCodeGen (SBV a)
-cgInput nm = do r <- cgSym forall_
-                sw <- cgSBVToSW r
-                modify' (\s -> s { cgInputs = (nm, CgAtomic sw) : cgInputs s })
+cgInput :: SymVal a => String -> SBVCodeGen (SBV a)
+cgInput nm = do r <- forall_
+                sv <- sbvToSymSV r
+                modify' (\s -> s { cgInputs = (nm, CgAtomic sv) : cgInputs s })
                 return r
 
 -- | Creates an array input in the generated code.
-cgInputArr :: SymWord a => Int -> String -> SBVCodeGen [SBV a]
+cgInputArr :: SymVal a => Int -> String -> SBVCodeGen [SBV a]
 cgInputArr sz nm
   | sz < 1 = error $ "SBV.cgInputArr: Array inputs must have at least one element, given " ++ show sz ++ " for " ++ show nm
-  | True   = do rs <- cgSym $ mapM (const forall_) [1..sz]
-                sws <- mapM cgSBVToSW rs
+  | True   = do rs <- mapM (const forall_) [1..sz]
+                sws <- mapM sbvToSymSV rs
                 modify' (\s -> s { cgInputs = (nm, CgArray sws) : cgInputs s })
                 return rs
 
 -- | Creates an atomic output in the generated code.
 cgOutput :: String -> SBV a -> SBVCodeGen ()
-cgOutput nm v = do _ <- cgSym (output v)
-                   sw <- cgSBVToSW v
-                   modify' (\s -> s { cgOutputs = (nm, CgAtomic sw) : cgOutputs s })
+cgOutput nm v = do _ <- output v
+                   sv <- sbvToSymSV v
+                   modify' (\s -> s { cgOutputs = (nm, CgAtomic sv) : cgOutputs s })
 
 -- | Creates an array output in the generated code.
-cgOutputArr :: SymWord a => String -> [SBV a] -> SBVCodeGen ()
+cgOutputArr :: SymVal a => String -> [SBV a] -> SBVCodeGen ()
 cgOutputArr nm vs
   | sz < 1 = error $ "SBV.cgOutputArr: Array outputs must have at least one element, received " ++ show sz ++ " for " ++ show nm
-  | True   = do _ <- cgSym (mapM output vs)
-                sws <- mapM cgSBVToSW vs
+  | True   = do mapM_ output vs
+                sws <- mapM sbvToSymSV vs
                 modify' (\s -> s { cgOutputs = (nm, CgArray sws) : cgOutputs s })
   where sz = length vs
 
 -- | Creates a returned (unnamed) value in the generated code.
 cgReturn :: SBV a -> SBVCodeGen ()
-cgReturn v = do _ <- cgSym (output v)
-                sw <- cgSBVToSW v
-                modify' (\s -> s { cgReturns = CgAtomic sw : cgReturns s })
+cgReturn v = do _ <- output v
+                sv <- sbvToSymSV v
+                modify' (\s -> s { cgReturns = CgAtomic sv : cgReturns s })
 
 -- | Creates a returned (unnamed) array value in the generated code.
-cgReturnArr :: SymWord a => [SBV a] -> SBVCodeGen ()
+cgReturnArr :: SymVal a => [SBV a] -> SBVCodeGen ()
 cgReturnArr vs
   | sz < 1 = error $ "SBV.cgReturnArr: Array returns must have at least one element, received " ++ show sz
-  | True   = do _ <- cgSym (mapM output vs)
-                sws <- mapM cgSBVToSW vs
+  | True   = do mapM_ output vs
+                sws <- mapM sbvToSymSV vs
                 modify' (\s -> s { cgReturns = CgArray sws : cgReturns s })
   where sz = length vs
 
@@ -329,9 +335,9 @@ instance Show CgPgmBundle where
 
 -- | Generate code for a symbolic program, returning a Code-gen bundle, i.e., collection
 -- of makefiles, source code, headers, etc.
-codeGen :: CgTarget l => l -> CgConfig -> String -> SBVCodeGen () -> IO (CgConfig, CgPgmBundle)
+codeGen :: CgTarget l => l -> CgConfig -> String -> SBVCodeGen a -> IO (a, CgConfig, CgPgmBundle)
 codeGen l cgConfig nm (SBVCodeGen comp) = do
-   (((), st'), res) <- runSymbolic CodeGen $ runStateT comp initCgState { cgFinalConfig = cgConfig }
+   ((retVal, st'), res) <- runSymbolic CodeGen $ runStateT comp initCgState { cgFinalConfig = cgConfig }
    let st = st' { cgInputs       = reverse (cgInputs st')
                 , cgOutputs      = reverse (cgOutputs st')
                 }
@@ -340,7 +346,7 @@ codeGen l cgConfig nm (SBVCodeGen comp) = do
    unless (null dupNames) $
         error $ "SBV.codeGen: " ++ show nm ++ " has following argument names duplicated: " ++ unwords dupNames
 
-   return (cgFinalConfig st, translate l (cgFinalConfig st) nm st res)
+   return (retVal, cgFinalConfig st, translate l (cgFinalConfig st) nm st res)
 
 -- | Render a code-gen bundle to a directory or to stdout
 renderCgPgmBundle :: Maybe FilePath -> (CgConfig, CgPgmBundle) -> IO ()
