@@ -483,33 +483,18 @@ getAssertionStackDepth = queryAssertionStackDepth <$> getQueryState
 -- | Upon a pop, we need to restore all arrays and tables. See: http://github.com/LeventErkok/sbv/issues/374
 restoreTablesAndArrays :: (MonadIO m, MonadQuery m) => m ()
 restoreTablesAndArrays = do st <- queryState
-                            qs <- getQueryState
 
-                            case queryTblArrPreserveIndex qs of
-                              Nothing       -> return ()
-                              Just (tc, ac) -> do tCount <- M.size  <$> (io . readIORef) (rtblMap   st)
-                                                  aCount <- IM.size <$> (io . readIORef) (rArrayMap st)
+                            tCount <- M.size  <$> (io . readIORef) (rtblMap   st)
+                            aCount <- IM.size <$> (io . readIORef) (rArrayMap st)
 
-                                                  let tInits = [ "table"  ++ show i ++ "_initializer" | i <- [tc .. tCount - 1]]
-                                                      aInits = [ "array_" ++ show i ++ "_initializer" | i <- [ac .. aCount - 1]]
-                                                      inits  = tInits ++ aInits
+                            let tInits = [ "table"  ++ show i ++ "_initializer" | i <- [0 .. tCount - 1]]
+                                aInits = [ "array_" ++ show i ++ "_initializer" | i <- [0 .. aCount - 1]]
+                                inits  = tInits ++ aInits
 
-                                                  case inits of
-                                                    []  -> return ()   -- Nothing to do
-                                                    [x] -> send True $ "(assert " ++ x ++ ")"
-                                                    xs  -> send True $ "(assert (and " ++ unwords xs ++ "))"
-
--- | Upon a push, record the cut-off point for table and array restoration, if we haven't already
-recordTablesAndArrayCutOff :: (MonadIO m, MonadQuery m) => m ()
-recordTablesAndArrayCutOff = do st <- queryState
-                                qs <- getQueryState
-
-                                case queryTblArrPreserveIndex qs of
-                                  Just _  -> return () -- already recorded, nothing to do
-                                  Nothing -> do tCount <- M.size  <$> (io . readIORef) (rtblMap   st)
-                                                aCount <- IM.size <$> (io . readIORef) (rArrayMap st)
-
-                                                modifyQueryState $ \s -> s {queryTblArrPreserveIndex = Just (tCount, aCount)}
+                            case inits of
+                              []  -> return ()   -- Nothing to do
+                              [x] -> send True $ "(assert " ++ x ++ ")"
+                              xs  -> send True $ "(assert (and " ++ unwords xs ++ "))"
 
 -- | Generalization of 'Data.SBV.Control.inNewAssertionStack'
 inNewAssertionStack :: (MonadIO m, MonadQuery m) => m a -> m a
@@ -524,7 +509,6 @@ push i
  | i <= 0 = error $ "Data.SBV: push requires a strictly positive level argument, received: " ++ show i
  | True   = do depth <- getAssertionStackDepth
                send True $ "(push " ++ show i ++ ")"
-               recordTablesAndArrayCutOff
                modifyQueryState $ \s -> s{queryAssertionStackDepth = depth + i}
 
 -- | Generalization of 'Data.SBV.Control.pop'
@@ -575,7 +559,10 @@ caseSplit printCases cases = do cfg <- getConfig
 -- | Generalization of 'Data.SBV.Control.resetAssertions'
 resetAssertions :: (MonadIO m, MonadQuery m) => m ()
 resetAssertions = do send True "(reset-assertions)"
-                     modifyQueryState $ \s -> s{queryAssertionStackDepth = 0}
+                     modifyQueryState $ \s -> s{ queryAssertionStackDepth = 0 }
+
+                     -- Make sure we restore tables and arrays after resetAssertions: See: https://github.com/LeventErkok/sbv/issues/535
+                     restoreTablesAndArrays
 
 -- | Generalization of 'Data.SBV.Control.echo'
 echo :: (MonadIO m, MonadQuery m) => String -> m ()
