@@ -19,6 +19,7 @@ module Data.SBV.Core.AlgReals (
            , mkPolyReal
            , algRealToSMTLib2
            , algRealToHaskell
+           , algRealToRational
            , mergeAlgReals
            , isExactRational
            , algRealStructuralEqual
@@ -32,6 +33,8 @@ import Data.Function   (on)
 import System.Random
 import Test.QuickCheck (Arbitrary(..))
 
+import Numeric (readSigned, readFloat)
+
 -- | Algebraic reals. Note that the representation is left abstract. We represent
 -- rational results explicitly, while the roots-of-polynomials are represented
 -- implicitly by their defining equation
@@ -187,7 +190,34 @@ algRealToSMTLib2 (AlgPolyRoot (i, AlgRealPoly xs) _) = "(root-obj (+ " ++ unword
 -- standard Haskell type that can represent root-of-polynomial variety.
 algRealToHaskell :: AlgReal -> String
 algRealToHaskell (AlgRational True r) = "((" ++ show r ++ ") :: Rational)"
-algRealToHaskell r                    = error $ "SBV.algRealToHaskell: Unsupported argument: " ++ show r
+algRealToHaskell r                    = error $ unlines [ ""
+                                                        , "SBV.algRealToHaskell: Unsupported argument:"
+                                                        , ""
+                                                        , "   " ++ show r
+                                                        , ""
+                                                        , "represents an irrational number, and cannot be converted to a Haskell value."
+                                                        ]
+
+-- | Convert an 'AlgReal' to a 'Rational'. If the 'AlgReal' is exact, then you get a 'Left' value. Otherwise,
+-- you get a 'Right' value which is simply an approximation.
+algRealToRational :: AlgReal -> Either Rational Rational
+algRealToRational a = case a of
+                        AlgRational True  r        -> Left r
+                        AlgRational False r        -> Left r
+                        AlgPolyRoot _     Nothing  -> bad
+                        AlgPolyRoot _     (Just s) -> let trimmed = case reverse s of
+                                                                     '.':'.':'.':rest -> reverse rest
+                                                                     _                -> s
+                                                      in case readSigned readFloat trimmed of
+                                                           [(v, "")] -> Right v
+                                                           _         -> bad
+   where bad = error $ unlines [ ""
+                               , "SBV.algRealToRational: Unsupported argument:"
+                               , ""
+                               , "   " ++ show a
+                               , ""
+                               , "represents an irrational number that cannot be approximated."
+                               ]
 
 -- Try to show a rational precisely if we can, with finite number of
 -- digits. Otherwise, show it as a rational value.
