@@ -33,7 +33,7 @@ module Data.SBV.Core.Symbolic
   , Op(..), PBOp(..), OvOp(..), FPOp(..), StrOp(..), SeqOp(..), SetOp(..), RegExp(..)
   , Quantifier(..), needsExistentials
   , RoundingMode(..)
-  , SBVType(..), svUninterpreted, newUninterpreted, addAxiom
+  , SBVType(..), svUninterpreted, newUninterpreted
   , SVal(..)
   , svMkSymVar, sWordN, sWordN_, sIntN, sIntN_
   , ArrayContext(..), ArrayInfo
@@ -577,7 +577,6 @@ data QueryState = QueryState { queryAsk                 :: Maybe Int -> String -
                              , queryTerminate           :: IO ()
                              , queryTimeOutValue        :: Maybe Int
                              , queryAssertionStackDepth :: Int
-                             , queryTblArrPreserveIndex :: Maybe (Int, Int)
                              }
 
 -- | Computations which support query operations.
@@ -1391,9 +1390,9 @@ introduceUserName st isTracker nmOrig k q sv = do
                                                            , "Only existential variables are supported in query mode."
                                                            ]
                    if isTracker
-                      then modifyState st rinps (second ((:) (sv, nm)) *** Set.insert nm)
+                      then modifyState st rinps (second ((sv, nm) :) *** Set.insert nm)
                                      $ noInteractive ["Adding a new tracker variable in interactive mode: " ++ show nm]
-                      else modifyState st rinps (first ((:) (q, (sv, nm))) *** Set.insert nm)
+                      else modifyState st rinps (first ((q, (sv, nm)) :) *** Set.insert nm)
                                      $ modifyIncState st rNewInps newInp
                    return $ SVal k $ Right $ cache (const (return sv))
 
@@ -1402,16 +1401,6 @@ introduceUserName st isTracker nmOrig k q sv = do
          -- die way ahead of that happening if that's the case!
          mkUnique prefix names = head $ dropWhile (`Set.member` names) (prefix : [prefix ++ "_" ++ show i | i <- [(0::Int)..]])
 
--- | Generalization of 'Data.SBV.addAxiom'
-addAxiom :: MonadSymbolic m => String -> [String] -> m ()
-addAxiom nm ax = do
-        st <- symbolicEnv
-        liftIO $ modifyState st raxioms ((nm, ax) :)
-                           $ noInteractive [ "Adding a new axiom:"
-                                           , "  Named: " ++ show nm
-                                           , "  Axiom: " ++ unlines ax
-                                           ]
-
 -- | Generalization of 'Data.SBV.runSymbolic'
 runSymbolic :: MonadIO m => SBVRunMode -> SymbolicT m a -> m (a, Result)
 runSymbolic currentRunMode (SymbolicT c) = do
