@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Provers.ABC
--- Copyright   :  (c) Adam Foltzer
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Provers.ABC
+-- Copyright : (c) Adam Foltzer
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- The connection to the ABC verification and synthesis tool
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Data.SBV.Provers.ABC(abc) where
 
 import Data.SBV.Core.Data
@@ -23,6 +25,7 @@ abc :: SMTSolver
 abc = SMTSolver {
            name         = ABC
          , executable   = "abc"
+         , preprocess   = id
          , options      = const ["-S", "%blast; &sweep -C 5000; &syn4; &cec -s -m -C 2000"]
          , engine       = standardEngine "SBV_ABC" "SBV_ABC_OPTIONS"
          , capabilities = SolverCapabilities {
@@ -32,10 +35,12 @@ abc = SMTSolver {
                               , supportsReals              = False
                               , supportsApproxReals        = False
                               , supportsIEEE754            = False
+                              , supportsSets               = False
                               , supportsOptimization       = False
                               , supportsPseudoBooleans     = False
                               , supportsCustomQueries      = False
                               , supportsGlobalDecls        = False
-                              , supportsFlattenedSequences = Nothing
+                              , supportsDataTypes          = False
+                              , supportsFlattenedModels    = Nothing
                               }
          }
