@@ -28,7 +28,7 @@ cvc4 = SMTSolver {
            name         = CVC4
          , executable   = "cvc4"
          , preprocess   = clean
-         , options      = const ["--lang", "smt", "--incremental", "--interactive", "--no-interactive-prompt"]
+         , options      = const ["--lang", "smt", "--incremental", "--interactive", "--no-interactive-prompt", "--model-witness-value"]
          , engine       = standardEngine "SBV_CVC4" "SBV_CVC4_OPTIONS"
          , capabilities = SolverCapabilities {
                                 supportsQuantifiers        = True
