@@ -1,16 +1,18 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Provers.Yices
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Provers.Yices
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- The connection to the Yices SMT solver
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE ScopedTypeVariables #-}
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Data.SBV.Provers.Yices(yices) where
 
 import Data.SBV.Core.Data
@@ -23,6 +25,7 @@ yices :: SMTSolver
 yices = SMTSolver {
            name         = Yices
          , executable   = "yices-smt2"
+         , preprocess   = id
          , options      = const ["--incremental"]
          , engine       = standardEngine "SBV_YICES" "SBV_YICES_OPTIONS"
          , capabilities = SolverCapabilities {
@@ -32,10 +35,12 @@ yices = SMTSolver {
                               , supportsReals              = True
                               , supportsApproxReals        = False
                               , supportsIEEE754            = False
+                              , supportsSets               = False
                               , supportsOptimization       = False
                               , supportsPseudoBooleans     = False
                               , supportsCustomQueries      = True
                               , supportsGlobalDecls        = False
-                              , supportsFlattenedSequences = Nothing
+                              , supportsDataTypes          = False
+                              , supportsFlattenedModels    = Nothing
                               }
          }
