@@ -1,16 +1,18 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Provers.MathSAT
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Provers.Z3
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- The connection to the MathSAT SMT solver
 -----------------------------------------------------------------------------
 
 {-# LANGUAGE ScopedTypeVariables #-}
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Data.SBV.Provers.Z3(z3) where
 
 import Data.SBV.Core.Data
@@ -23,6 +25,7 @@ z3 :: SMTSolver
 z3 = SMTSolver {
            name         = Z3
          , executable   = "z3"
+         , preprocess   = id
          , options      = modConfig ["-nw", "-in", "-smt2"]
          , engine       = standardEngine "SBV_Z3" "SBV_Z3_OPTIONS"
          , capabilities = SolverCapabilities {
@@ -32,12 +35,15 @@ z3 = SMTSolver {
                               , supportsReals              = True
                               , supportsApproxReals        = True
                               , supportsIEEE754            = True
+                              , supportsSets               = True
                               , supportsOptimization       = True
                               , supportsPseudoBooleans     = True
                               , supportsCustomQueries      = True
                               , supportsGlobalDecls        = True
-                              , supportsFlattenedSequences = Just [ "(set-option :pp.max_depth      4294967295)"
+                              , supportsDataTypes          = True
+                              , supportsFlattenedModels    = Just [ "(set-option :pp.max_depth      4294967295)"
                                                                   , "(set-option :pp.min_alias_size 4294967295)"
+                                                                  , "(set-option :model.inline_def  true      )"
                                                                   ]
                               }
          }
