@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Tools.GenTest
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Tools.GenTest
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test generation from symbolic programs
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Data.SBV.Tools.GenTest (
         -- * Test case generation
         genTest, TestVectors, getTestValues, renderTest, TestStyle(..)
@@ -25,13 +27,15 @@ import Data.SBV.Core.Data
 
 import Data.SBV.Utils.PrettyNum
 
+import qualified Data.Foldable as F (toList)
+
 -- | Type of test vectors (abstract)
-newtype TestVectors = TV [([CW], [CW])]
+newtype TestVectors = TV [([CV], [CV])]
 
 -- | Retrieve the test vectors for further processing. This function
 -- is useful in cases where 'renderTest' is not sufficient and custom
 -- output (or further preprocessing) is needed.
-getTestValues :: TestVectors -> [([CW], [CW])]
+getTestValues :: TestVectors -> [([CV], [CV])]
 getTestValues (TV vs) = vs
 
 -- | Generate a set of concrete test values from a symbolic program. The output
@@ -44,9 +48,9 @@ genTest n m = gen 0 []
          | i == n = return $ TV $ reverse sofar
          | True   = do t <- tc
                        gen (i+1) (t:sofar)
-        tc = do (_, Result {resTraces=tvals, resConsts=cs, resConstraints=cstrs, resOutputs=os}) <- runSymbolic Concrete (m >>= output)
+        tc = do (_, Result {resTraces=tvals, resConsts=cs, resConstraints=cstrs, resOutputs=os}) <- runSymbolic (Concrete Nothing) (m >>= output)
                 let cval = fromMaybe (error "Cannot generate tests in the presence of uninterpeted constants!") . (`lookup` cs)
-                    cond = and [cwToBool (cval v) | (False, _, v) <- cstrs] -- Only pick-up "hard" constraints, as indicated by False in the fist component
+                    cond = and [cvToBool (cval v) | (False, _, v) <- F.toList cstrs] -- Only pick-up "hard" constraints, as indicated by False in the fist component
                 if cond
                    then return (map snd tvals, map cval os)
                    else tc   -- try again, with the same set of constraints
@@ -64,7 +68,7 @@ renderTest (Haskell n)    (TV vs) = haskell n vs
 renderTest (C n)          (TV vs) = c       n vs
 renderTest (Forte n b ss) (TV vs) = forte   n b ss vs
 
-haskell :: String -> [([CW], [CW])] -> String
+haskell :: String -> [([CV], [CV])] -> String
 haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do not edit!"
                                       , ""
                                       , "module " ++ modName ++ "(" ++ n ++ ") where"
@@ -89,13 +93,13 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
                 needsInt     = any isSW params
                 needsWord    = any isUW params
                 needsRatio   = any isR params
-                isR cw       = case kindOf cw of
+                isR cv       = case kindOf cv of
                                  KReal -> True
                                  _     -> False
-                isSW cw      = case kindOf cw of
+                isSW cv      = case kindOf cv of
                                  KBounded True _ -> True
                                  _               -> False
-                isUW cw      = case kindOf cw of
+                isUW cv      = case kindOf cv of
                                  KBounded False sz -> sz > 1
                                  _                 -> False
         modName = let (f:r) = n in toUpper f : r
@@ -103,7 +107,7 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
         getType []         = "[a]"
         getType ((i, o):_) = "[(" ++ mapType typeOf i ++ ", " ++ mapType typeOf o ++ ")]"
         mkLine  (i, o)     = "("  ++ mapType valOf  i ++ ", " ++ mapType valOf  o ++ ")"
-        mapType f cws = mkTuple $ map f $ groupBy ((==) `on` kindOf) cws
+        mapType f cvs = mkTuple $ map f $ groupBy ((==) `on` kindOf) cvs
         mkTuple [x] = x
         mkTuple xs  = "(" ++ intercalate ", " xs ++ ")"
         typeOf []    = "()"
@@ -113,39 +117,44 @@ haskell vname vs = intercalate "\n" $ [ "-- Automatically generated by SBV. Do n
         valOf  [x]   = s x
         valOf  xs    = "[" ++ intercalate ", " (map s xs) ++ "]"
 
-        t cw = case kindOf cw of
-                 KBool             -> "Bool"
-                 KBounded False 8  -> "Word8"
-                 KBounded False 16 -> "Word16"
-                 KBounded False 32 -> "Word32"
-                 KBounded False 64 -> "Word64"
-                 KBounded True  8  -> "Int8"
-                 KBounded True  16 -> "Int16"
-                 KBounded True  32 -> "Int32"
-                 KBounded True  64 -> "Int64"
-                 KUnbounded        -> "Integer"
-                 KFloat            -> "Float"
-                 KDouble           -> "Double"
-                 KChar             -> error "SBV.renderTest: Unsupported char"
-                 KString           -> error "SBV.renderTest: Unsupported string"
-                 KReal             -> error $ "SBV.renderTest: Unsupported real valued test value: " ++ show cw
-                 KList es          -> error $ "SBV.renderTest: Unsupported list valued test: [" ++ show es ++ "]"
-                 KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                 _                 -> error $ "SBV.renderTest: Unexpected CW: " ++ show cw
+        t cv = case kindOf cv of
+                 KBool               -> "Bool"
+                 KBounded False 8    -> "Word8"
+                 KBounded False 16   -> "Word16"
+                 KBounded False 32   -> "Word32"
+                 KBounded False 64   -> "Word64"
+                 KBounded True  8    -> "Int8"
+                 KBounded True  16   -> "Int16"
+                 KBounded True  32   -> "Int32"
+                 KBounded True  64   -> "Int64"
+                 KUnbounded          -> "Integer"
+                 KFloat              -> "Float"
+                 KDouble             -> "Double"
+                 KChar               -> error "SBV.renderTest: Unsupported char"
+                 KString             -> error "SBV.renderTest: Unsupported string"
+                 KReal               -> error $ "SBV.renderTest: Unsupported real valued test value: " ++ show cv
+                 KList es            -> error $ "SBV.renderTest: Unsupported list valued test: [" ++ show es ++ "]"
+                 KSet  es            -> error $ "SBV.renderTest: Unsupported set valued test: {" ++ show es ++ "}"
+                 KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                 _                   -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
-        s cw = case kindOf cw of
-                  KBool             -> take 5 (show (cwToBool cw) ++ repeat ' ')
-                  KBounded sgn   sz -> let CWInteger w = cwVal cw in shex  False True (sgn, sz) w
-                  KUnbounded        -> let CWInteger w = cwVal cw in shexI False True           w
-                  KFloat            -> let CWFloat w   = cwVal cw in showHFloat w
-                  KDouble           -> let CWDouble w  = cwVal cw in showHDouble w
-                  KChar             -> error "SBV.renderTest: Unsupported char"
-                  KString           -> error "SBV.renderTest: Unsupported string"
-                  KReal             -> let CWAlgReal w = cwVal cw in algRealToHaskell w
-                  KList es          -> error $ "SBV.renderTest: Unsupported list valued sort: [" ++ show es ++ "]"
-                  KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+        s cv = case kindOf cv of
+                  KBool               -> take 5 (show (cvToBool cv) ++ repeat ' ')
+                  KBounded sgn   sz   -> let CInteger w = cvVal cv in shex  False True (sgn, sz) w
+                  KUnbounded          -> let CInteger w = cvVal cv in shexI False True           w
+                  KFloat              -> let CFloat   w = cvVal cv in showHFloat w
+                  KDouble             -> let CDouble  w = cvVal cv in showHDouble w
+                  KChar               -> error "SBV.renderTest: Unsupported char"
+                  KString             -> error "SBV.renderTest: Unsupported string"
+                  KReal               -> let CAlgReal w = cvVal cv in algRealToHaskell w
+                  KList es            -> error $ "SBV.renderTest: Unsupported list valued sort: [" ++ show es ++ "]"
+                  KSet  es            -> error $ "SBV.renderTest: Unsupported set valued sort: {" ++ show es ++ "}"
+                  KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                  k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple: " ++ show k
+                  k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe: " ++ show k
+                  k@KEither{}         -> error $ "SBV.renderTest: Unsupported sum: " ++ show k
 
-c :: String -> [([CW], [CW])] -> String
+c :: String -> [([CV], [CV])] -> String
 c n vs = intercalate "\n" $
               [ "/* Automatically generated by SBV. Do not edit! */"
               , ""
@@ -207,50 +216,64 @@ c n vs = intercalate "\n" $
               , "  return 0;"
               , "}"
               ]
-  where mkField p cw i = "    " ++ t ++ " " ++ p ++ show i ++ ";"
-            where t = case kindOf cw of
-                        KBool             -> "SBool"
-                        KBounded False 8  -> "SWord8"
-                        KBounded False 16 -> "SWord16"
-                        KBounded False 32 -> "SWord32"
-                        KBounded False 64 -> "SWord64"
-                        KBounded True  8  -> "SInt8"
-                        KBounded True  16 -> "SInt16"
-                        KBounded True  32 -> "SInt32"
-                        KBounded True  64 -> "SInt64"
-                        KFloat            -> "SFloat"
-                        KDouble           -> "SDouble"
-                        KChar             -> error "SBV.renderTest: Unsupported char"
-                        KString           -> error "SBV.renderTest: Unsupported string"
-                        KUnbounded        -> error "SBV.renderTest: Unbounded integers are not supported when generating C test-cases."
-                        KReal             -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
-                        KUserSort us _    -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                        _                 -> error $ "SBV.renderTest: Unexpected CW: " ++ show cw
+  where mkField p cv i = "    " ++ t ++ " " ++ p ++ show i ++ ";"
+            where t = case kindOf cv of
+                        KBool               -> "SBool"
+                        KBounded False 8    -> "SWord8"
+                        KBounded False 16   -> "SWord16"
+                        KBounded False 32   -> "SWord32"
+                        KBounded False 64   -> "SWord64"
+                        KBounded True  8    -> "SInt8"
+                        KBounded True  16   -> "SInt16"
+                        KBounded True  32   -> "SInt32"
+                        KBounded True  64   -> "SInt64"
+                        k@KBounded{}        -> error $ "SBV.renderTest: Unsupported kind: " ++ show k
+                        KFloat              -> "SFloat"
+                        KDouble             -> "SDouble"
+                        KChar               -> error "SBV.renderTest: Unsupported char"
+                        KString             -> error "SBV.renderTest: Unsupported string"
+                        KUnbounded          -> error "SBV.renderTest: Unbounded integers are not supported when generating C test-cases."
+                        KReal               -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+                        KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                        k@KList{}           -> error $ "SBV.renderTest: Unsupported list sort: "   ++ show k
+                        k@KSet{}            -> error $ "SBV.renderTest: Unsupported set sort: "   ++ show k
+                        k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple sort: "  ++ show k
+                        k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe sort: "  ++ show k
+                        k@KEither{}         -> error $ "SBV.renderTest: Unsupported either sort: " ++ show k
+
+
         mkLine (is, os) = "{{" ++ intercalate ", " (map v is) ++ "}, {" ++ intercalate ", " (map v os) ++ "}}"
-        v cw = case kindOf cw of
-                  KBool           -> if cwToBool cw then "true " else "false"
-                  KBounded sgn sz -> let CWInteger w = cwVal cw in chex  False True (sgn, sz) w
-                  KUnbounded      -> let CWInteger w = cwVal cw in shexI False True           w
-                  KFloat          -> let CWFloat w   = cwVal cw in showCFloat w
-                  KDouble         -> let CWDouble w  = cwVal cw in showCDouble w
-                  KChar           -> error "SBV.renderTest: Unsupported char"
-                  KString         -> error "SBV.renderTest: Unsupported string"
-                  k@KList{}       -> error $ "SBV.renderTest: Unsupported list sort!" ++ show k
-                  KUserSort us _  -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
-                  KReal           -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+
+        v cv = case kindOf cv of
+                  KBool               -> if cvToBool cv then "true " else "false"
+                  KBounded sgn sz     -> let CInteger w = cvVal cv in chex  False True (sgn, sz) w
+                  KUnbounded          -> let CInteger w = cvVal cv in shexI False True           w
+                  KFloat              -> let CFloat w   = cvVal cv in showCFloat w
+                  KDouble             -> let CDouble w  = cvVal cv in showCDouble w
+                  KChar               -> error "SBV.renderTest: Unsupported char"
+                  KString             -> error "SBV.renderTest: Unsupported string"
+                  k@KList{}           -> error $ "SBV.renderTest: Unsupported list sort!" ++ show k
+                  k@KSet{}            -> error $ "SBV.renderTest: Unsupported set sort!" ++ show k
+                  KUninterpreted us _ -> error $ "SBV.renderTest: Unsupported uninterpreted sort: " ++ us
+                  KReal               -> error "SBV.renderTest: Real values are not supported when generating C test-cases."
+                  k@KTuple{}          -> error $ "SBV.renderTest: Unsupported tuple sort!" ++ show k
+                  k@KMaybe{}          -> error $ "SBV.renderTest: Unsupported maybe sort!" ++ show k
+                  k@KEither{}         -> error $ "SBV.renderTest: Unsupported sum sort!" ++ show k
+
         outLine
           | null vs = "printf(\"\");"
           | True    = "printf(\"%*d. " ++ fmtString ++ "\\n\", " ++ show (length (show (length vs - 1))) ++ ", i"
                     ++ concatMap ("\n           , " ++ ) (zipWith inp is [(0::Int)..] ++ zipWith out os [(0::Int)..])
                     ++ ");"
           where (is, os) = head vs
-                inp cw i = mkBool cw (n ++ "[i].input.i"  ++ show i)
-                out cw i = mkBool cw (n ++ "[i].output.o" ++ show i)
-                mkBool cw s = case kindOf cw of
+                inp cv i = mkBool cv (n ++ "[i].input.i"  ++ show i)
+                out cv i = mkBool cv (n ++ "[i].output.o" ++ show i)
+                mkBool cv s = case kindOf cv of
                                 KBool -> "(" ++ s ++ " == true) ? \"true \" : \"false\""
                                 _     -> s
                 fmtString = unwords (map fmt is) ++ " -> " ++ unwords (map fmt os)
-        fmt cw = case kindOf cw of
+
+        fmt cv = case kindOf cv of
                     KBool             -> "%s"
                     KBounded False  8 -> "0x%02\"PRIx8\""
                     KBounded False 16 -> "0x%04\"PRIx16\"U"
@@ -266,9 +289,9 @@ c n vs = intercalate "\n" $
                     KString           -> error "SBV.renderTest: Unsupported string"
                     KUnbounded        -> error "SBV.renderTest: Unsupported unbounded integers for C generation."
                     KReal             -> error "SBV.renderTest: Unsupported real valued values for C generation."
-                    _                 -> error $ "SBV.renderTest: Unexpected CW: " ++ show cw
+                    _                 -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
 
-forte :: String -> Bool -> ([Int], [Int]) -> [([CW], [CW])] -> String
+forte :: String -> Bool -> ([Int], [Int]) -> [([CV], [CV])] -> String
 forte vname bigEndian ss vs = intercalate "\n" $ [ "// Automatically generated by SBV. Do not edit!"
                                              , "let " ++ n ++ " ="
                                              , "   let c s = val [_, r] = str_split s \"'\" in " ++ blaster
@@ -284,33 +307,41 @@ forte vname bigEndian ss vs = intercalate "\n" $ [ "// Automatically generated b
          | True      = "rev (map (\\s. s == \"1\") (explode (string_tl r)))"
         toF True  = '1'
         toF False = '0'
-        blast cw = let noForte w = error "SBV.renderTest: " ++ w ++ " values are not supported when generating Forte test-cases."
-                   in case kindOf cw of
-                       KBool             -> [toF (cwToBool cw)]
-                       KBounded False 8  -> xlt  8 (cwVal cw)
-                       KBounded False 16 -> xlt 16 (cwVal cw)
-                       KBounded False 32 -> xlt 32 (cwVal cw)
-                       KBounded False 64 -> xlt 64 (cwVal cw)
-                       KBounded True 8   -> xlt  8 (cwVal cw)
-                       KBounded True 16  -> xlt 16 (cwVal cw)
-                       KBounded True 32  -> xlt 32 (cwVal cw)
-                       KBounded True 64  -> xlt 64 (cwVal cw)
-                       KFloat            -> noForte "Float"
-                       KDouble           -> noForte "Double"
-                       KChar             -> noForte "Char"
-                       KString           -> noForte "String"
-                       KReal             -> noForte "Real"
-                       KList ek          -> noForte $ "List of " ++ show ek
-                       KUnbounded        -> noForte "Unbounded integers"
-                       _                 -> error $ "SBV.renderTest: Unexpected CW: " ++ show cw
-        xlt s (CWInteger  v)  = [toF (testBit v i) | i <- [s-1, s-2 .. 0]]
-        xlt _ (CWFloat    r)  = error $ "SBV.renderTest.Forte: Unexpected float value: "         ++ show r
-        xlt _ (CWDouble   r)  = error $ "SBV.renderTest.Forte: Unexpected double value: "        ++ show r
-        xlt _ (CWChar     r)  = error $ "SBV.renderTest.Forte: Unexpected char value: "          ++ show r
-        xlt _ (CWString   r)  = error $ "SBV.renderTest.Forte: Unexpected string value: "        ++ show r
-        xlt _ (CWAlgReal  r)  = error $ "SBV.renderTest.Forte: Unexpected real value: "          ++ show r
-        xlt _ CWList{}        = error   "SBV.renderTest.Forte: Unexpected list value!"
-        xlt _ (CWUserSort r)  = error $ "SBV.renderTest.Forte: Unexpected uninterpreted value: " ++ show r
+        blast cv = let noForte w = error "SBV.renderTest: " ++ w ++ " values are not supported when generating Forte test-cases."
+                   in case kindOf cv of
+                        KBool              -> [toF (cvToBool cv)]
+                        KBounded False 8   -> xlt  8 (cvVal cv)
+                        KBounded False 16  -> xlt 16 (cvVal cv)
+                        KBounded False 32  -> xlt 32 (cvVal cv)
+                        KBounded False 64  -> xlt 64 (cvVal cv)
+                        KBounded True 8    -> xlt  8 (cvVal cv)
+                        KBounded True 16   -> xlt 16 (cvVal cv)
+                        KBounded True 32   -> xlt 32 (cvVal cv)
+                        KBounded True 64   -> xlt 64 (cvVal cv)
+                        KFloat             -> noForte "Float"
+                        KDouble            -> noForte "Double"
+                        KChar              -> noForte "Char"
+                        KString            -> noForte "String"
+                        KReal              -> noForte "Real"
+                        KList ek           -> noForte $ "List of " ++ show ek
+                        KSet  ek           -> noForte $ "Set of " ++ show ek
+                        KUnbounded         -> noForte "Unbounded integers"
+                        KUninterpreted s _ -> noForte $ "Uninterpreted kind " ++ show s
+                        _                  -> error $ "SBV.renderTest: Unexpected CV: " ++ show cv
+
+        xlt s (CInteger  v)  = [toF (testBit v i) | i <- [s-1, s-2 .. 0]]
+        xlt _ (CFloat    r)  = error $ "SBV.renderTest.Forte: Unexpected float value: "         ++ show r
+        xlt _ (CDouble   r)  = error $ "SBV.renderTest.Forte: Unexpected double value: "        ++ show r
+        xlt _ (CChar     r)  = error $ "SBV.renderTest.Forte: Unexpected char value: "          ++ show r
+        xlt _ (CString   r)  = error $ "SBV.renderTest.Forte: Unexpected string value: "        ++ show r
+        xlt _ (CAlgReal  r)  = error $ "SBV.renderTest.Forte: Unexpected real value: "          ++ show r
+        xlt _ CList{}        = error   "SBV.renderTest.Forte: Unexpected list value!"
+        xlt _ CSet{}         = error   "SBV.renderTest.Forte: Unexpected set value!"
+        xlt _ CTuple{}       = error   "SBV.renderTest.Forte: Unexpected list value!"
+        xlt _ CMaybe{}       = error   "SBV.renderTest.Forte: Unexpected maybe value!"
+        xlt _ CEither{}      = error   "SBV.renderTest.Forte: Unexpected sum value!"
+        xlt _ (CUserSort r)  = error $ "SBV.renderTest.Forte: Unexpected uninterpreted value: " ++ show r
+
         mkLine  (i, o) = "("  ++ mkTuple (form (fst ss) (concatMap blast i)) ++ ", " ++ mkTuple (form (snd ss) (concatMap blast o)) ++ ")"
         mkTuple []  = "()"
         mkTuple [x] = x
