@@ -1,21 +1,23 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Data.SBV.Tools.Overflow
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Data.SBV.Tools.Overflow
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Implementation of overflow detection functions.
 -- Based on: <http://www.microsoft.com/en-us/research/wp-content/uploads/2016/02/z3prefix.pdf>
 -----------------------------------------------------------------------------
 
-{-# LANGUAGE FlexibleContexts     #-}
-{-# LANGUAGE FlexibleInstances    #-}
-{-# LANGUAGE ImplicitParams       #-}
-{-# LANGUAGE Rank2Types           #-}
-{-# LANGUAGE ScopedTypeVariables  #-}
-{-# LANGUAGE TypeSynonymInstances #-}
+{-# LANGUAGE FlexibleContexts    #-}
+{-# LANGUAGE FlexibleInstances   #-}
+{-# LANGUAGE ImplicitParams      #-}
+{-# LANGUAGE Rank2Types          #-}
+{-# LANGUAGE ScopedTypeVariables #-}
+{-# LANGUAGE TypeApplications    #-}
+
+{-# OPTIONS_GHC -Wall -Werror #-}
 
 module Data.SBV.Tools.Overflow (
 
@@ -31,17 +33,16 @@ import Data.SBV.Core.Data
 import Data.SBV.Core.Symbolic
 import Data.SBV.Core.Model
 import Data.SBV.Core.Operations
-import Data.SBV.Utils.Boolean
 
 import GHC.Stack
 
 import Data.Int
 import Data.Word
+import Data.Proxy
 
 -- Doctest only
 -- $setup
 -- >>> import Data.SBV.Provers.Prover (prove, allSat)
--- >>> import Data.SBV.Utils.Boolean ((<=>))
 
 -- | Detecting underflow/overflow conditions. For each function,
 -- the first result is the condition under which the computation
@@ -52,7 +53,7 @@ class ArithOverflow a where
   --
   -- A tell tale sign of unsigned addition overflow is when the sum is less than minumum of the arguments.
   --
-  -- >>> prove $ \x y -> snd (bvAddO x (y::SWord16)) <=> x + y .< x `smin` y
+  -- >>> prove $ \x y -> snd (bvAddO x (y::SWord16)) .<=> x + y .< x `smin` y
   -- Q.E.D.
   bvAddO :: a -> a -> (SBool, SBool)
 
@@ -81,7 +82,7 @@ class ArithOverflow a where
   -- | Bit-vector negation. Unsigned negation neither underflows nor overflows. Signed negation can only overflow, when the argument is
   -- @minBound@:
   --
-  -- >>> prove $ \x -> x .== minBound <=> snd (bvNegO (x::SInt16))
+  -- >>> prove $ \x -> x .== minBound .<=> snd (bvNegO (x::SInt16))
   -- Q.E.D.
   bvNegO :: a -> (SBool, SBool)
 
@@ -105,7 +106,7 @@ instance ArithOverflow SVal where
 -- | A class of checked-arithmetic operations. These follow the usual arithmetic,
 -- except make calls to 'Data.SBV.sAssert' to ensure no overflow/underflow can occur.
 -- Use them in conjunction with 'Data.SBV.safe' to ensure no overflow can happen.
-class (ArithOverflow (SBV a), Num a, SymWord a) => CheckedArithmetic a where
+class (ArithOverflow (SBV a), Num a, SymVal a) => CheckedArithmetic a where
   (+!)          :: (?loc :: CallStack) => SBV a -> SBV a -> SBV a
   (-!)          :: (?loc :: CallStack) => SBV a -> SBV a -> SBV a
   (*!)          :: (?loc :: CallStack) => SBV a -> SBV a -> SBV a
@@ -389,19 +390,19 @@ bvsnego n x = (underflow, overflow)
 -- (254 :: SWord8,(True,False))
 -- >>> sFromIntegralO (2 :: SInt16) :: (SWord8, (SBool, SBool))
 -- (2 :: SWord8,(False,False))
--- >>> prove $ \x -> sFromIntegralO (x::SInt32) .== (sFromIntegral x :: SInteger, (false, false))
+-- >>> prove $ \x -> sFromIntegralO (x::SInt32) .== (sFromIntegral x :: SInteger, (sFalse, sFalse))
 -- Q.E.D.
 --
 -- As the last example shows, converting to `sInteger` never underflows or overflows for any value.
-sFromIntegralO :: forall a b. (Integral a, HasKind a, Num a, SymWord a, HasKind b, Num b, SymWord b) => SBV a -> (SBV b, (SBool, SBool))
-sFromIntegralO x = case (kindOf x, kindOf (undefined :: b)) of
+sFromIntegralO :: forall a b. (Integral a, HasKind a, Num a, SymVal a, HasKind b, Num b, SymVal b) => SBV a -> (SBV b, (SBool, SBool))
+sFromIntegralO x = case (kindOf x, kindOf (Proxy @b)) of
                      (KBounded False n, KBounded False m) -> (res, u2u n m)
                      (KBounded False n, KBounded True  m) -> (res, u2s n m)
                      (KBounded True n,  KBounded False m) -> (res, s2u n m)
                      (KBounded True n,  KBounded True  m) -> (res, s2s n m)
                      (KUnbounded,       KBounded s m)     -> (res, checkBounds s m)
-                     (KBounded{},       KUnbounded)       -> (res, (false, false))
-                     (KUnbounded,       KUnbounded)       -> (res, (false, false))
+                     (KBounded{},       KUnbounded)       -> (res, (sFalse, sFalse))
+                     (KUnbounded,       KUnbounded)       -> (res, (sFalse, sFalse))
                      (kFrom,            kTo)              -> error $ "sFromIntegralO: Expected bounded-BV types, received: " ++ show (kFrom, kTo)
 
   where res :: SBV b
@@ -425,16 +426,16 @@ sFromIntegralO x = case (kindOf x, kindOf (undefined :: b)) of
 
         u2u :: Int -> Int -> (SBool, SBool)
         u2u n m = (underflow, overflow)
-          where underflow  = false
+          where underflow  = sFalse
                 overflow
-                  | n <= m = false
+                  | n <= m = sFalse
                   | True   = SBV $ svNot $ allZero (n-1) m x
 
         u2s :: Int -> Int -> (SBool, SBool)
         u2s n m = (underflow, overflow)
-          where underflow = false
+          where underflow = sFalse
                 overflow
-                  | m > n = false
+                  | m > n = sFalse
                   | True  = SBV $ svNot $ allZero (n-1) (m-1) x
 
         s2u :: Int -> Int -> (SBool, SBool)
@@ -442,26 +443,26 @@ sFromIntegralO x = case (kindOf x, kindOf (undefined :: b)) of
           where underflow = SBV $ (unSBV x `svTestBit` (n-1)) `svEqual` svTrue
 
                 overflow
-                  | m >= n - 1 = false
+                  | m >= n - 1 = sFalse
                   | True       = SBV $ svAll [(unSBV x `svTestBit` (n-1)) `svEqual` svFalse, svNot $ allZero (n-1) m x]
 
         s2s :: Int -> Int -> (SBool, SBool)
         s2s n m = (underflow, overflow)
           where underflow
-                  | m > n = false
+                  | m > n = sFalse
                   | True  = SBV $ svAll [(unSBV x `svTestBit` (n-1)) `svEqual` svTrue,  svNot $ allOne  (n-1) (m-1) x]
 
                 overflow
-                  | m > n = false
+                  | m > n = sFalse
                   | True  = SBV $ svAll [(unSBV x `svTestBit` (n-1)) `svEqual` svFalse, svNot $ allZero (n-1) (m-1) x]
 
 -- | Version of 'sFromIntegral' that has calls to 'Data.SBV.sAssert' for checking no overflow/underflow can happen. Use it with a 'Data.SBV.safe' call.
-sFromIntegralChecked :: forall a b. (?loc :: CallStack, Integral a, HasKind a, HasKind b, Num a, SymWord a, HasKind b, Num b, SymWord b) => SBV a -> SBV b
-sFromIntegralChecked x = sAssert (Just ?loc) (msg "underflows") (bnot u)
-                       $ sAssert (Just ?loc) (msg "overflows")  (bnot o)
+sFromIntegralChecked :: forall a b. (?loc :: CallStack, Integral a, HasKind a, HasKind b, Num a, SymVal a, HasKind b, Num b, SymVal b) => SBV a -> SBV b
+sFromIntegralChecked x = sAssert (Just ?loc) (msg "underflows") (sNot u)
+                       $ sAssert (Just ?loc) (msg "overflows")  (sNot o)
                          r
   where kFrom = show $ kindOf x
-        kTo   = show $ kindOf (undefined :: b)
+        kTo   = show $ kindOf (Proxy @b)
         msg c = "Casting from " ++ kFrom ++ " to " ++ kTo ++ " " ++ c
 
         (r, (u, o)) = sFromIntegralO x
@@ -486,18 +487,18 @@ signPick1 fu fs a
  | True      = let (u, o) = fu n a in (SBV u, SBV o)
  where n = intSizeOf a
 
-checkOp1 :: HasKind a => CallStack -> String -> (a -> SBV b) -> (a -> (SBool, SBool)) -> a -> SBV b
-checkOp1 loc w op cop a = sAssert (Just loc) (msg "underflows") (bnot u)
-                        $ sAssert (Just loc) (msg "overflows")  (bnot o)
+checkOp1 :: (HasKind a, HasKind b) => CallStack -> String -> (a -> SBV b) -> (a -> (SBool, SBool)) -> a -> SBV b
+checkOp1 loc w op cop a = sAssert (Just loc) (msg "underflows") (sNot u)
+                        $ sAssert (Just loc) (msg "overflows")  (sNot o)
                         $ op a
   where k = show $ kindOf a
         msg c = k ++ " " ++ w ++ " " ++ c
 
         (u, o) = cop a
 
-checkOp2 :: HasKind a => CallStack -> String -> (a -> b -> SBV c) -> (a -> b -> (SBool, SBool)) -> a -> b -> SBV c
-checkOp2 loc w op cop a b = sAssert (Just loc) (msg "underflows") (bnot u)
-                          $ sAssert (Just loc) (msg "overflows")  (bnot o)
+checkOp2 :: (HasKind a, HasKind c) => CallStack -> String -> (a -> b -> SBV c) -> (a -> b -> (SBool, SBool)) -> a -> b -> SBV c
+checkOp2 loc w op cop a b = sAssert (Just loc) (msg "underflows") (sNot u)
+                          $ sAssert (Just loc) (msg "overflows")  (sNot o)
                           $ a `op` b
   where k = show $ kindOf a
         msg c = k ++ " " ++ w ++ " " ++ c
