@@ -87,11 +87,13 @@ instance Show a => Show (Range a) where
 -- [(0.0,oo)]
 -- >>> ranges $ \x -> x .< (0::SReal)
 -- [(-oo,0.0)]
-ranges :: forall a. (Ord a, Num a, SymVal a,  SMTValue a, SatModel a, Metric a, SymVal (MetricSpace a), SatModel (MetricSpace a)) => (SBV a -> SBool) -> IO [Range a]
+-- >>> ranges $ \(x :: SWord8) -> 2*x .== 4
+-- [[2,3),(129,130]]
+ranges :: forall a. (Ord a, Num a, SymVal a,  SatModel a, Metric a, SymVal (MetricSpace a), SatModel (MetricSpace a)) => (SBV a -> SBool) -> IO [Range a]
 ranges = rangesWith defaultSMTCfg
 
 -- | Compute ranges, using the given solver configuration.
-rangesWith :: forall a. (Ord a, Num a, SymVal a,  SMTValue a, SatModel a, Metric a, SymVal (MetricSpace a), SatModel (MetricSpace a)) => SMTConfig -> (SBV a -> SBool) -> IO [Range a]
+rangesWith :: forall a. (Ord a, Num a, SymVal a,  SatModel a, Metric a, SymVal (MetricSpace a), SatModel (MetricSpace a)) => SMTConfig -> (SBV a -> SBool) -> IO [Range a]
 rangesWith cfg prop = do mbBounds <- getInitialBounds
                          case mbBounds of
                            Nothing -> return []
@@ -135,45 +137,68 @@ rangesWith cfg prop = do mbBounds <- getInitialBounds
                                                                     Just r  -> r
                                  _                             -> error $ "Data.SBV.ranges.getRegVal: Cannot parse " ++ show cv
 
-            IndependentResult m <- optimizeWith cfg Independent $ do x <- free_
-                                                                     constrain $ prop x
-                                                                     minimize "min" x
-                                                                     maximize "max" x
-            case head (map snd m) of
-              Unsatisfiable{} -> return Nothing
-              Unknown{}       -> error "Solver said Unknown!"
-              ProofError{}    -> error (show (IndependentResult m))
-              _               -> let Just (Just mi) = getModelObjectiveValue "min" `fmap` ("min" `lookup` m)
-                                     Just (Just ma) = getModelObjectiveValue "max" `fmap` ("max" `lookup` m)
-                                 in return $ Just $ Range (getGenVal mi) (getGenVal ma)
 
+                getBound cstr = do let objName = "boundValue"
+                                   res@(LexicographicResult m) <- optimizeWith cfg Lexicographic $ do x <- free_
+                                                                                                      constrain $ prop x
+                                                                                                      cstr objName x
+                                   case m of
+                                     Unsatisfiable{} -> return Nothing
+                                     Unknown{}       -> error "Solver said Unknown!"
+                                     ProofError{}    -> error (show res)
+                                     _               -> return $ getModelObjectiveValue objName m
+
+            mi <- getBound minimize
+            ma <- getBound maximize
+            case (mi, ma) of
+              (Just minV, Just maxV) -> return $ Just $ Range (getGenVal minV) (getGenVal maxV)
+              _                      -> return Nothing
+
+        -- Is this range satisfiable? Returns a witness to it.
+        witness :: Range a -> Symbolic (SBV a)
+        witness (Range lo hi) = do x :: SBV a <- free_
+
+                                   let restrict v open closed = case v of
+                                                                  Unbounded -> sTrue
+                                                                  Open   a  -> x `open`   literal a
+                                                                  Closed a  -> x `closed` literal a
+
+                                       lower = restrict lo (.>) (.>=)
+                                       upper = restrict hi (.<) (.<=)
+
+                                   constrain $ lower .&& upper
+
+                                   return x
+
+        isFeasible :: Range a -> IO Bool
+        isFeasible r = runSMTWith cfg $ do _ <- witness r
+
+                                           query $ do cs <- checkSat
+                                                      case cs of
+                                                        Unsat -> return False
+                                                        Unk   -> error "Data.SBV.interval.isFeasible: Solver said unknown!"
+                                                        Sat   -> return True
 
         bisect :: Range a -> IO (Maybe [Range a])
-        bisect (Range lo hi) = runSMTWith cfg $ do
-                                     x <- free_
+        bisect r@(Range lo hi) = runSMTWith cfg $ do x <- witness r
 
-                                     let restrict v open closed = case v of
-                                                                    Unbounded -> sTrue
-                                                                    Open   a  -> x `open`   literal a
-                                                                    Closed a  -> x `closed` literal a
+                                                     constrain $ sNot (prop x)
 
-                                         lower = restrict lo (.>) (.>=)
-                                         upper = restrict hi (.<) (.<=)
-
-                                     constrain $ lower .&& upper .&& sNot (prop x)
-
-                                     query $ do cs <- checkSat
-                                                case cs of
-                                                  Unsat -> return Nothing
-                                                  Unk   -> error "Data.SBV.interval.bisect: Solver said unknown!"
-                                                  Sat   -> do midV <- Open <$> getValue x
-                                                              return $ Just [Range lo midV, Range midV hi]
+                                                     query $ do cs <- checkSat
+                                                                case cs of
+                                                                  Unsat -> return Nothing
+                                                                  Unk   -> error "Data.SBV.interval.bisect: Solver said unknown!"
+                                                                  Sat   -> do midV <- Open <$> getValue x
+                                                                              return $ Just [Range lo midV, Range midV hi]
 
         search :: [Range a] -> [Range a] -> IO [Range a]
         search []     sofar = return $ reverse sofar
-        search (c:cs) sofar = do mbCS <- bisect c
-                                 case mbCS of
-                                   Nothing  -> search cs          (c:sofar)
-                                   Just xss -> search (xss ++ cs) sofar
+        search (c:cs) sofar = do feasible <- isFeasible c
+                                 if feasible
+                                    then do mbCS <- bisect c
+                                            case mbCS of
+                                              Nothing  -> search cs          (c:sofar)
+                                              Just xss -> search (xss ++ cs) sofar
+                                    else search cs sofar
 
 {-# ANN rangesWith ("HLint: ignore Replace case with fromMaybe" :: String) #-}
