@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Documentation.SBV.Examples.Uninterpreted.Function
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : Documentation.SBV.Examples.Uninterpreted.Function
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Demonstrates function counter-examples
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Documentation.SBV.Examples.Uninterpreted.Function where
 
 import Data.SBV
@@ -22,4 +24,4 @@ f = uninterpret "f"
 -- >>> prove thmGood
 -- Q.E.D.
 thmGood :: SWord8 -> SWord8 -> SWord8 -> SBool
-thmGood x y z = x .== y+2 ==> f x z .== f (y + 2) z
+thmGood x y z = x .== y+2 .=> f x z .== f (y + 2) z
