@@ -17,12 +17,12 @@ module BenchSuite.Puzzles.Coins(benchmarks) where
 import Documentation.SBV.Examples.Puzzles.Coins
 
 import Utils.SBVBenchFramework
-import BenchSuite.Overhead.SBVOverhead
+import BenchSuite.Bench.Bench as S
 
 
 -- benchmark suite
 benchmarks :: Runner
-benchmarks = runner "Coins" coinsPgm
+benchmarks = S.run "Coins" coinsPgm
   where coinsPgm = do cs <- mapM mkCoin [1..6]
                       mapM_ constrain [c s | s <- combinations cs, length s >= 2, c <- [c1, c2, c3, c4, c5, c6]]
                       constrain $ sAnd $ zipWith (.>=) cs (tail cs)
