@@ -1,3 +1,16 @@
+-----------------------------------------------------------------------------
+-- |
+-- Module    : SBVHLint
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
+--
+-- HLint interface for SBV testsuite
+-----------------------------------------------------------------------------
+
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module Main (main) where
 
 import Utils.SBVTestFramework (getTestEnvironment, TestEnvironment(..))
@@ -10,6 +23,7 @@ arguments =
     [ "Data"
     , "SBVTestSuite"
     , "-i", "Use otherwise"
+    , "-i", "Parse error"
     ]
 
 main :: IO ()
@@ -21,7 +35,7 @@ main = do
     case testEnv of
       TestEnvLocal   -> runHLint
       TestEnvCI{}    -> if testPercentage < 50
-                           then do putStrLn $ "Test percentage below tresheold, skipping hlint: " ++ show testPercentage
+                           then do putStrLn $ "Test percentage below threshold, skipping hlint: " ++ show testPercentage
                                    exitSuccess
                            else runHLint
       TestEnvUnknown -> do putStrLn "Unknown test environment, skipping hlint run"
