@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.Basics.QRem
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.Basics.QRem
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for Examples.Basics.QRem
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.Basics.QRem(tests) where
 
 import Utils.SBVTestFramework
@@ -27,7 +29,7 @@ tests =
 -- defined to be 0 and the remainder is the numerator
 qrem :: (Num a, EqSymbolic a, SDivisible a) => a -> a -> SBool
 qrem x y = ite (y .== 0)
-               ((0, x) .== (q, r) &&& (0, x) .== (d, m))
-               (x .== y * q + r &&& x .== y * d + m)
+               ((0, x) .== (q, r) .&& (0, x) .== (d, m))
+               (x .== y * q + r .&& x .== y * d + m)
   where (q, r) = x `sQuotRem` y
         (d, m) = x `sDivMod` y
