@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.CodeGeneration.Floats
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.CodeGeneration.Floats
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test-suite for generating floating-point related C code
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.CodeGeneration.Floats(tests) where
 
 import Data.SBV.Internals
@@ -20,7 +22,9 @@ tests :: TestTree
 tests = testGroup "CodeGeneration.Floats" [
    goldenVsStringShow "floats_cgen" code
  ]
- where code  = snd <$> compileToCLib' "floatCodeGen" cases
+ where code  = thd <$> compileToCLib' "floatCodeGen" cases
+
+       thd (_, _, r) = r
 
        setup = do cgSRealType CgLongDouble
                   cgIntegerSize 64
