@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.Optimization.Combined
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.Optimization.Combined
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for optimization routines, combined objectives
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.Optimization.Combined(tests) where
 
 import Utils.SBVTestFramework
@@ -48,7 +50,7 @@ combined2 = do a <- sBool "a"
                assertWithPenalty "soft_c" c (Penalty 3 (Just "A"))
 
                constrain $ a .== c
-               constrain $ bnot (a &&& b)
+               constrain $ sNot (a .&& b)
 
 pareto1 :: Goal
 pareto1 = do x <- sInteger "x"
