@@ -1,14 +1,16 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.Puzzles.PowerSet
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.Puzzles.PowerSet
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for Examples.Puzzles.PowerSet
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.Puzzles.PowerSet(tests) where
 
 import Utils.SBVTestFramework
@@ -21,5 +23,5 @@ tests =
 pSet :: Int -> IO Bool
 pSet n = do cnt <- numberOfModels $ do mapM_ (\i -> sBool ("e" ++ show i)) [1..n]
                                        -- Look ma! No constraints!
-                                       return (true :: SBool)
+                                       return sTrue
             return (cnt == 2^n)
