@@ -1,10 +1,10 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  TestSuite.Uninterpreted.Sort
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.Uninterpreted.Sort
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
 -- Test suite for uninterpreted sorts
 -----------------------------------------------------------------------------
@@ -12,6 +12,8 @@
 {-# LANGUAGE DeriveDataTypeable  #-}
 {-# LANGUAGE ScopedTypeVariables #-}
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.Uninterpreted.Sort(tests) where
 
 import Utils.SBVTestFramework
@@ -25,10 +27,10 @@ tests =
     ]
 
 data L = Nil | Cons Int L deriving (Eq, Ord, Data, Read, Show)
-instance SymWord L
+instance SymVal L
 instance HasKind L
 instance SatModel L where
-  parseCWs = undefined -- make GHC shut up about the missing method, we won't actually call it
+  parseCVs = undefined -- make GHC shut up about the missing method, we won't actually call it
 
 type UList = SBV L
 
@@ -41,5 +43,5 @@ p0 = do
     constrain $ len l0 .== 0
     constrain $ len l1 .== 1
     x :: SInteger <- symbolic "x"
-    constrain $ x .== 0 ||| x.== 1
-    return $ l .== l0 ||| l .== l1
+    constrain $ x .== 0 .|| x.== 1
+    return $ l .== l0 .|| l .== l1
