@@ -1,14 +1,15 @@
 -----------------------------------------------------------------------------
 -- |
--- Module      :  Examples.TestSuite.Uninterpreted
--- Copyright   :  (c) Levent Erkok
--- License     :  BSD3
--- Maintainer  :  erkokl@gmail.com
--- Stability   :  experimental
+-- Module    : TestSuite.Uninterpreted.Uninterpreted
+-- Copyright : (c) Levent Erkok
+-- License   : BSD3
+-- Maintainer: erkokl@gmail.com
+-- Stability : experimental
 --
--- Test suite for Examples.Uninterpreted.Uninterpreted
 -----------------------------------------------------------------------------
 
+{-# OPTIONS_GHC -Wall -Werror #-}
+
 module TestSuite.Uninterpreted.Uninterpreted(tests) where
 
 import Utils.SBVTestFramework
@@ -29,10 +30,10 @@ g :: SInt8 -> SWord16 -> SWord32
 g = uninterpret "g"
 
 p0 :: SInt8 -> SInt8 -> SBool
-p0 x y   = x .== y ==> f x .== f y      -- OK
+p0 x y   = x .== y .=> f x .== f y      -- OK
 
 p1 :: SInt8 -> SWord16 -> SWord16 -> SBool
-p1 x y z = y .== z ==> g x y .== g x z  -- OK
+p1 x y z = y .== z .=> g x y .== g x z  -- OK
 
 p2 :: SInt8 -> SWord16 -> SWord16 -> SBool
-p2 x y z = y .== z ==> g x y .== f x    -- Not true
+p2 x y z = y .== z .=> g x y .== f x    -- Not true
